<?php
namespace Teknisa\PLA\Modules\Cardapio\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;
use \Setforms2\Shared\ORM\DateTime;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DTO;

class Pla00100ElaboracaoDoCardapioService {

    protected $dataSourceManager;
    protected $environment;
    protected $entityManager;
    protected $novoCodigo;
    protected $auth;
    protected $tiposDeCardapio = array('descentralizado' => 'S', 'livre' => 'N');
    const PARAMGERAL = 'paramgeral';
    const PARAMFILIAL = 'paramfilial';
    const PARAMPLANEJA = 'paramplaneja';
    const SERVICO = 'servico';
    const COMENSAIS_DIA_CARDAPIO = 'comensais_dia_cardapio';

    public function __construct($dataSourceManager, $environment, $entityManager, $novoCodigo, $crudService, $auth) {
        $this->dataSourceManager = $dataSourceManager;
        $this->environment       = $environment;
        $this->entityManager     = $entityManager;
        $this->novoCodigo        = $novoCodigo;
        $this->crudService       = $crudService;
        $this->auth              = $auth;
    }

    public function retornaTiposDePrato($filterCriteria, $cdfilial, $cdservfili, $dtrefecnfmes) {

        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $filterCriteria->addCondition('CDSERVFILI', $cdservfili);
        $filterCriteria->addCondition('DTREFECNFMES', $dtrefecnfmes);
        return $this->dataSourceManager->findBy($filterCriteria);

    }

    public function retornaParamgeral() {

        $filterCriteria = new FilterCriteria(self::PARAMGERAL);
        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();

        return $rows[0];
    }

    public function retornaDataProcessamento($cdfilial) {

        $dataSet = $this->dataSourceManager->findBy(new FilterCriteria(self::PARAMGERAL));

        $rows = $dataSet->getRows();

        if ($rows[0]['IDUTILDTPROC'] != 'S') {

            $dataProcessamento = date('d/m/Y');

        } else {

            $dataProcessamento = $this->retornaDataProcPorFilial($cdfilial);

        }

        return $dataProcessamento;

    }

    public function retornaDataProcPorFilial($cdfilial) {

        $filterCriteria = new FilterCriteria(self::PARAMFILIAL);
        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);

        $rows = $dataSet->getRows();

        $dataProcessamento = $rows[0]['DTPROCESSA'];

        if (empty($dataProcessamento)) {

            $dataProcessamento = date('d/m/Y');
        }

        return $dataProcessamento;

    }

    /**
     * Método que retorna a data de processamento somada com os dias de bloquio do cardápio.
     * Caso o dia seja zero, será possível alterar da data encontrada para frente.
     * Casa o dia seja maior que zero, será considerada como primeiro dia, a data de processamento.
     *
     * @param String $cdfilial
     * @param String|Int $diasDeBloqueio
     *
     * @return DateTime
     */
    public function retornaDataProcBloqueio($cdfilial, $diasDeBloqueio) {

        $dataProcessamento = $this->retornaDataProcessamento($cdfilial);

        $dias = 'P' . $diasDeBloqueio . 'D';
        $explodeDate = explode(' ', $dataProcessamento);
        $explodeDate = explode('/', $explodeDate[0]);
        $dataProcBloqueio = new DateTime($explodeDate[2].'-'.$explodeDate[1].'-'.$explodeDate[0]);
        $dataProcBloqueio->add(new \DateInterval($dias));

        return $dataProcBloqueio;

    }

    /**
     * Método que retorna o tipo de operador, sendo ele MATRIZ ou FILIAL.
     *
     * @return String
     */
    public function retornaTipoOperador() {
        $row = array(
            'CDOPERADOR'   => $this->environment->getCdOperador(),
            'AUTENTICACAO' => array(
    			'COLUMN_NAME'=> null,
    			'ALIAS_NAME'=> 'LRCD'
		    )
        );
        $permission = $this->auth->returnOperatorInGroup($row);

        return $permission ? 'MATRIZ' : 'FILIAL';

    }
    
    public function retornaOperadorGrupoRevisao() {
        $row = array(
            'CDOPERADOR'   => $this->environment->getCdOperador(),
            'AUTENTICACAO' => array(
    			'COLUMN_NAME'=> null,
    			'ALIAS_NAME'=> 'ARRV'
		    )
        );
        $permission = $this->auth->returnOperatorInGroup($row);
        
        return $permission ? 'S' : 'N';
    }

    /**
     * Método que retorna o grupo de operadores que pode realizar a revisão do cardápio.
     *
     * @return String|null
     */
    public function retornaGrupoRevisaoCardapio() {

        $filterCriteria = new FilterCriteria('autgrupop');
        $filterCriteria->addCondition('IDAUTGRUPOP', 'LRCD');

        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();

        return isset($rows[0]) ? $rows[0]['CDGRUPOPER'] : null;

    }
    
    public function retornaGrupoRevisaoReceita() {
        $filterCriteria = new FilterCriteria('autgrupop');
        $filterCriteria->addCondition('IDAUTGRUPOP', 'ARRV');
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();
        return isset($rows[0]) ? $rows[0]['CDGRUPOPER'] : null;
    }

    /**
     * Método que retorna qual a situação em que o cardápio se encontra, podendo ser
     * LIBERADO_FILIAL ou NAO_LIBERADO_FILIAL.
     *
     * @return String
     */
    public function retornaEstadoDoCardapio($cdfilial, $cdservfili, $dtrefecnfmes) {

        $filterCriteria = new FilterCriteria('ctrlaprovcard');
        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $filterCriteria->addCondition('CDSERVFILI', $cdservfili);
        $filterCriteria->addCondition('DTMESCARDAPROV', $dtrefecnfmes);

        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();

        if (isset($rows[0])) {

            if (isset($rows[0]['DTAPROCARDMATR'])){

                $estadoDoCardapio = 'APROVADO_MATRIZ';

            } else if (isset($rows[0]['DTAPROCARDFILI'])) {

                $estadoDoCardapio = 'APROVADO_FILIAL';

            } else if(isset($rows[0]['DTLIBECARDMATR'])) {

                $estadoDoCardapio = 'LIBERADO_FILIAL';

            } else {

                $estadoDoCardapio = 'NAO_LIBERADO_FILIAL';
            }

        } else {
            $estadoDoCardapio = 'NAO_LIBERADO_FILIAL';
        }

        return $estadoDoCardapio;

    }

    public function retornaTipoDeVisualizacaoDescentralizada($cdfilial, $dtrefecnfmes, $estadoDoCardapio, $tipoOperador) {
        $filterCriteria = new FilterCriteria('calencard');
        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $filterCriteria->addCondition('DTMESCALENCARD', $dtrefecnfmes);

        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();
        if (isset($rows[0]) && isset($rows[0]['DTPREVLIBECARD'])) {
            if (($estadoDoCardapio == 'APROVADO_MATRIZ' && $tipoOperador == 'FILIAL') || ($estadoDoCardapio == 'LIBERADO_FILIAL' && $tipoOperador == 'MATRIZ')) {
                $tipoDeVisualizacao = 'CONSULTA'; // Tolentino esteve aqui
            } else {
                if ($tipoOperador == 'MATRIZ') {
                    $tipoDeVisualizacao = $this->validaPeriodoOperadorMatriz($rows[0], $estadoDoCardapio);
                } else {
                    $tipoDeVisualizacao = $this->validaPeriodoOperadorFilial($rows[0], $estadoDoCardapio);
                }
            }
        } else {
            throw new \Exception("Cardápio não possui calendário.", 1);
        }
        return $tipoDeVisualizacao;
    }

    public function validaPeriodoOperadorMatriz($row, $estadoDoCardapio) {

        $dataAtual = new DateTime(date('Y/m/d'));
        $qtdDias = $row['QTDIAPRORAPROV'] ? $row['QTDIAPRORAPROV'] : "0";
        $dias = 'P' . $qtdDias . 'D'; 
        $dataLimiteMatriz = explode(' ', $row['DTLIMAPROVMATR']);
        $dataLimiteMatriz = $this->dateToDateTime($dataLimiteMatriz[0]);
        $dataLimiteMatriz->add(new \DateInterval($dias));

        $dataLimiteFilial = explode(' ', $row['DTLIMAPROVFILI']);
        $dataLimiteFilial = $this->dateToDateTime($dataLimiteFilial[0]);
        $dataLimiteFilial->add(new \DateInterval($dias));

        if($estadoDoCardapio == 'APROVADO_MATRIZ'){
            
            $tipoDeVisualizacao = 'ALTERACAO';
            
        }else if ($estadoDoCardapio == 'NAO_LIBERADO_FILIAL') {

            $tipoDeVisualizacao = 'ALTERACAO';

        } else if ($estadoDoCardapio == 'LIBERADO_FILIAL' && $dataAtual <= $dataLimiteFilial) {

            $tipoDeVisualizacao = 'CONSULTA';

        } else if ($estadoDoCardapio == 'LIBERADO_FILIAL' && $dataAtual > $dataLimiteFilial) {

            $tipoDeVisualizacao = 'BLOQUEADO';

        } else if ($estadoDoCardapio == 'APROVADO_FILIAL') {

            $tipoDeVisualizacao = 'ALTERACAO';

        } else {

            $tipoDeVisualizacao = 'CONSULTA';
        }

        return $tipoDeVisualizacao;

    }

    public function validaPeriodoOperadorFilial($row, $estadoDoCardapio) {

        $dataAtual = new DateTime(date('Y/m/d'));
        $qtdDias = $row['QTDIAPRORAPROV'] ? $row['QTDIAPRORAPROV'] : "0";
        $dias = 'P' . $qtdDias . 'D';
        $dataLimiteFilial = explode(' ', $row['DTLIMAPROVFILI']);
        $dataLimiteFilial = $this->dateToDateTime($dataLimiteFilial[0]);
        $dataLimiteFilial->add(new \DateInterval($dias));

        if ($estadoDoCardapio == 'NAO_LIBERADO_FILIAL') {

            $tipoDeVisualizacao = 'CONSULTA';

        } else if ($estadoDoCardapio == 'LIBERADO_FILIAL' && $dataAtual <= $dataLimiteFilial) {

            $tipoDeVisualizacao = 'ALTERACAO';

        } else if ($estadoDoCardapio == 'LIBERADO_FILIAL' && $dataAtual > $dataLimiteFilial) {

            $tipoDeVisualizacao = 'BLOQUEADO';

            throw new \Exception("Seu cardápio está bloqueado!", 1);

        } else {

            $tipoDeVisualizacao = 'CONSULTA';
        }

        return $tipoDeVisualizacao;

    }

    public function retornaTipoDeVisualizacaoLivre($cdfilial, $tipoOperador) {
        if($this->tipoDeCardapio($cdfilial) == 'D'){
            if($tipoOperador == 'FILIAL'){
                $filterCriteria = new FilterCriteria(self::PARAMGERAL);
                $dataSet2 = $this->dataSourceManager->findBy($filterCriteria);
                $rowParamGeral = $dataSet2->getRows();
                
                if($rowParamGeral[0]['IDPEALTCARDSERV'] == 'N'){
                    $tipoDeVisualizacao = 'CONSULTA';
                } else {
                    $tipoDeVisualizacao = 'ALTERACAO';
                } 
                
            } else {
                $tipoDeVisualizacao = 'ALTERACAO';
            }
        }else{
            $tipoDeVisualizacao = 'ALTERACAO';
        }

        return $tipoDeVisualizacao;
    }
    
    /*
    * Método que retorna o tipo de cardapio de acordo com a função implementada no delphi que valida apenas a filial
    * 
    * @return String
    */
    public function tipoDeCardapio($cdfilial){
        $filterServico = new FilterCriteria(self::SERVICO);
        $filterServico->addCondition('CDFILIAL', $cdfilial);
        $filterServico->addCondition('IDCARDDESCENTR', 'S');
        
        $datasetServico = $this->dataSourceManager->findBy($filterServico);
        $rowsServico = $datasetServico->getRows();
        
        $filterParamGeral = new FilterCriteria(self::PARAMGERAL);
        $dataSet2 = $this->dataSourceManager->findBy($filterParamGeral);
        $rowParamGeral = $dataSet2->getRows();
        
        if(isset($rowParamGeral[0]['IDUTCARDPADR']) && $rowParamGeral[0]['IDUTCARDPADR'] === 'N'){
            return $rowParamGeral[0]['IDUTCARDPADR'];
        }else{
            if(empty($datasetServico)){
                 return $rowParamGeral[0]['IDUTCARDPADR'];
            }else{
                return 'D';
            }
        }
        
    }

    /**
     * Método que retorna o tipo do cardápio, sendo ele descentralizado (S) ou livre (N).
     *
     * @return String
     */
    public function retornaTipoDeCardapio($cdfilial, $cdservfili) {

        $filterCriteria1 = new FilterCriteria(self::SERVICO);
        $filterCriteria1->addCondition('CDFILIAL', $cdfilial);
        $filterCriteria1->addCondition('CDSERVFILI', $cdservfili);

        $dataSet1 = $this->dataSourceManager->findBy($filterCriteria1);
        $rowsServico = $dataSet1->getRows();
        
        $filterCriteria2 = new FilterCriteria(self::PARAMGERAL);
        $dataSet2 = $this->dataSourceManager->findBy($filterCriteria2);
        $rowParamGeral = $dataSet2->getRows();
        
        if(isset($rowParamGeral[0]['IDUTCARDPADR']) && $rowParamGeral[0]['IDUTCARDPADR'] === 'N'){
            return $rowParamGeral[0]['IDUTCARDPADR'];
        }else{
            return $rowsServico[0]['IDCARDDESCENTR'];
        }

    }

    public function retornaComensaisDiasCardapio($cdfilial, $cdservfili, $dataInicial) {

        $dataFinal = $this->retornaUltimaDataMesString($dataInicial, 'd/m/Y');

        $filterCriteria = new FilterCriteria(self::COMENSAIS_DIA_CARDAPIO);
        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $filterCriteria->addCondition('CDSERVFILI', $cdservfili);
        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
        $filterCriteria->setWhereClause("DTCARDAPIO BETWEEN TO_DATE('{$dataInicial}', 'DD/MM/YYYY') AND TO_DATE('{$dataFinal}', 'DD/MM/YYYY')");

        $dataSet = $this->dataSourceManager->findBy($filterCriteria);

        return $dataSet;

    }

    public function retornaParamplaneja($cdfilial) {

        $filterCriteria = new FilterCriteria(self::PARAMPLANEJA);
        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();
        return $rows[0];
    }

    public function retornaDiasFeriadoFilial($filterCriteria, $cdfilial, $dataInicial) {

        $dataFinal = $this->retornaUltimaDataMesString($dataInicial, 'd/m/Y');
        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $filterCriteria->setWhereClause("DTFERIFILI BETWEEN TO_DATE('{$dataInicial}', 'DD/MM/YYYY') AND TO_DATE('{$dataFinal}', 'DD/MM/YYYY')");
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();

        return $rows;

    }

    public function retornaDiasFeriadoCardapio($filterCriteria, $cdfilial, $dataInicial) {

        $dataFinal = $this->retornaUltimaDataMesString($dataInicial, 'd/m/Y');
        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $filterCriteria->setWhereClause("DTFERICARD BETWEEN TO_DATE('{$dataInicial}', 'DD/MM/YYYY') AND TO_DATE('{$dataFinal}', 'DD/MM/YYYY')");
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();

        return $rows;

    }

    public function retornaUltimaDataMesString($dataInicial, $format) {

        $explodeDate = explode('/', $dataInicial);
        $dataReferencia = $explodeDate[2].'-'.$explodeDate[1].'-'.$explodeDate[0];
        $objetoDataReferencia = new \DateTime(date("Y-m-t", strtotime($dataReferencia)));

        return $objetoDataReferencia->format($format);

    }

    public function retornaComensaisEstimadosServico($filterCriteria, $cdfilial, $cdservfili) {

        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $filterCriteria->addCondition('CDSERVFILI', $cdservfili);

        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        $rows = $dataSet->getRows();
        return $rows[0];
    }

    public function organizaDataSetComensais($comensaisServico, $diaDaSemana) {
        $comensaisEstimados = 'QTCOMESTI' . $diaDaSemana;
        $clientesEstimados = 'QTCLIESTI' . $diaDaSemana;
        $funcionariosEstimados = 'QTFUNESTI' . $diaDaSemana;
        $doacoesEstimadas = 'QTDOAESTI' . $diaDaSemana;

        $comensais = array();

        $comensais[] = array(
            'QTCOMESTICAR' => isset($comensaisServico[$comensaisEstimados]) ? $comensaisServico[$comensaisEstimados] : "0",
            'QTCLIESTICAR' => isset($comensaisServico[$clientesEstimados]) ? $comensaisServico[$clientesEstimados] : "0",
            'QTFUNESTICAR' => isset($comensaisServico[$funcionariosEstimados]) ? $comensaisServico[$funcionariosEstimados] : "0",
            'QTDOAESTICAR' => isset($comensaisServico[$doacoesEstimadas]) ? $comensaisServico[$doacoesEstimadas] : "0",
        );
        return $comensais;
    }

    public function retornaColunaSemana($diaDaSemana) {
        $diasDaSemana = array(
            '0' => 'DOM','1' => 'SEG',
            '2' => 'TER','3' => 'QUA',
            '4' => 'QUI','5' => 'SEX',
            '6' => 'SAB','7' => 'FER',
        );
        return $diasDaSemana[$diaDaSemana];
    }

    public function updateCardadia($row) {

        $params = array(
            'nrcardapio' => $row['NRCARDAPIO'],
            'cdfilial' => $row['CDFILIAL'],
            'nrorg' => $this->environment->getNrOrg(),
        );

        $cardadia = $this->entityManager->getRepository('Planejamento\Entities\Cardadia')->findBy($params);

        $cardadia[0]->setQtcomesticar($row['QTCOMESTICAR']);
        $cardadia[0]->setQtcliesticar($row['QTCLIESTICAR']);
        $cardadia[0]->setQtfunesticar($row['QTFUNESTICAR']);
        $cardadia[0]->setQtdoaesticar($row['QTDOAESTICAR']);
        $cardadia[0]->setQtestiturno1($row['QTESTITURNO1']);
        $cardadia[0]->setQtestiturno2($row['QTESTITURNO2']);
        $cardadia[0]->setQtestiturno3($row['QTESTITURNO3']);
        $cardadia[0]->setQtestiturno4($row['QTESTITURNO4']);
        $cardadia[0]->setDscardapio($row['DSCARDAPIO']);

        $this->entityManager->persist($cardadia[0]);
        $this->entityManager->flush();
    }

    public function insertCardadia($row) {

        $cardadia = new \Planejamento\Entities\Cardadia();

        $cardadia = $this->buildCardadia($cardadia, $row);

        $this->entityManager->persist($cardadia);
        $this->entityManager->flush();

        return $cardadia->getNrcardapio();

    }

    public function buildCardadia($cardadia, $row) {

        $nrcardapio = $this->novoCodigo->getCodeByTableName('CARDAPIO'.$row['CDFILIAL'], $this->environment->getNrOrg());
        $dscardapio = isset($row['DSCARDAPIO']) ? $row['DSCARDAPIO'] : null;
        $cardadia->setCdfilial($row['CDFILIAL']);
        $cardadia->setNrcardapio($nrcardapio);
        $cardadia->setCdservfili($row['CDSERVFILI']);
        $cardadia->setCdoperador($this->environment->getCdOperador());
        $cardadia->setDtcardapio($this->dateToDateTime($row['DTCARDAPIO']));
        $cardadia->setQtcomesticar($row['QTCOMESTICAR']);
        $cardadia->setQtcliesticar($row['QTCLIESTICAR']);
        $cardadia->setQtfunesticar($row['QTFUNESTICAR']);
        $cardadia->setQtdoaesticar($row['QTDOAESTICAR']);
        $cardadia->setQtestiturno1($row['QTESTITURNO1']);
        $cardadia->setQtestiturno2($row['QTESTITURNO2']);
        $cardadia->setQtestiturno3($row['QTESTITURNO3']);
        $cardadia->setQtestiturno4($row['QTESTITURNO4']);
        $cardadia->setDscardapio($dscardapio);
        $cardadia->setDtmescard($this->firstDayOfMonth($row['DTCARDAPIO']));
        $cardadia->setNrorg($this->environment->getNrOrg());
        $cardadia->setDtinclusao(new \DateTime());
        $cardadia->setDtultatu(new \DateTime());
        $cardadia->setCdoperinclusao($this->environment->getCdOperador());

        return $cardadia;
    }

    public function salvaPercentualPrato($row) {

        $params = array(
            'nrcardapio' => $row['NRCARDAPIO'],
            'cdfilial' => $row['CDFILIAL'],
            'nrorg' => $this->environment->getNrOrg(),
            'nrseqpratcar' => $row['NRSEQCNFMES'],
        );

        $prato = $this->entityManager->getRepository('Planejamento\Entities\Compcard')->findBy($params);

        $prato[0]->setVrpeconscard($row['VRPECONSCARD']);

        $this->entityManager->persist($prato[0]);
        $this->entityManager->flush();

    }

    public function salvaCardacli($row, $nrcardapio) {
        $params = array(
            'nrcardapio' => $nrcardapio,
            'cdfilial' => $row['CDFILIAL'],
            'nrorg' => $this->environment->getNrOrg(),
            'cdcliente' => $row['CDCLIENTE'],
            'cddestfili' => $row['CDDESTFILI'],
        );
        $cardacli = $this->entityManager->getRepository('Planejamento\Entities\Cardacli')->findBy($params);
        if (isset($cardacli[0])) {
            $cardacli[0]->setQtcomesticli($row['QTCOMESTICLI']);
            $this->entityManager->persist($cardacli[0]);
            $this->entityManager->flush();
        } else {
            $row['NRCARDAPIO'] = $nrcardapio;
            $this->insertCardacli($row);
        }
    }

    public function insertCardacli($row) {
        $row['CDOPERINCLUSAO'] = $this->environment->getCdOperador();
        $row['CDOPERULTATU'] = $this->environment->getCdOperador();
        $row['DTINCLUSAO'] = date('d/m/Y');
        $row['DTULTATU'] = date('d/m/Y');
        $row['IDATIVO'] = "S";
        $row['NRORG'] = $this->environment->getNrOrg();
        $row['NRORGINCLUSAO'] = $this->environment->getNrOrg();
        $row['NRORGULTATU'] = $this->environment->getNrOrg();
        $row['__is_new'] = true;
        $this->crudService->save('cardacli', array($row));
    }
    
    public function dateToDateTime($date) {

        $explode = explode('/', $date);

        return new DateTime($explode[2].'-'.$explode[1].'-'.$explode[0]);
    }

    public function firstDayOfMonth($date) {

        $explode = explode('/', $date);

        return new DateTime(date($explode[2].'-'.$explode[1].'-01'));
    }

    public function salvaPrato($row) {

        $params = array(
            'nrcardapio' => $row['NRCARDAPIO'],
            'cdfilial' => $row['CDFILIAL'],
            'nrorg' => $this->environment->getNrOrg(),
            'nrseqpratcar' => $row['NRSEQPRATCAR'],
        );

        $prato = $this->entityManager->getRepository('Planejamento\Entities\Compcard')->findBy($params);

        if (isset($prato[0])) {

            $prato[0]->setVrpeconscard($row['VRPECONSCARD']);
            $prato[0]->setCdprato($row['CDPRATO']);
            $prato[0]->setCdfilauxprat($row['CDFILAUXPRAT']);
            $prato[0]->setCdpratomat($row['CDPRATOMAT']);
            $prato[0]->setCdfilauxpratmat($row['CDFILAUXPRATMAT']);
            $prato[0]->setIdpratoespe($row['IDPRATOESPE']);

            $this->entityManager->persist($prato[0]);
            $this->entityManager->flush();

        } else {

            $this->insertCompcard($row);
        }

    }

    public function insertCompcard($row) {

        $compcard = new \Planejamento\Entities\Compcard();

        $compcard = $this->buildCompcard($compcard, $row);
        $this->entityManager->persist($compcard);
        $this->entityManager->flush();

    }

    public function buildCompcard($compcard, $row) {

        $compcard->setCdfilial($row['CDFILIAL']);
        $compcard->setNrcardapio($row['NRCARDAPIO']);
        $compcard->setNrseqpratcar($row['NRSEQPRATCAR']);
        $compcard->setCdfilauxprat($row['CDFILAUXPRAT']);
        $compcard->setCdprato($row['CDPRATO']);
        $compcard->setVrpeconscard($row['VRPECONSCARD']);
        $compcard->setCdtipoprat($row['CDTIPOPRAT']);
        $compcard->setNrordpratcar($row['NRORDPRATCAR']);
        $compcard->setNrorg($this->environment->getNrOrg());
        $compcard->setIdpratoespe($row['IDPRATOESPE']);
        $compcard->setDtinclusao(new \DateTime());
        $compcard->setDtultatu(new \DateTime());
        $compcard->setCdoperinclusao($this->environment->getCdOperador());

        return $compcard;
    }

    public function excluirPrato($row) {

        $params = array(
            'nrcardapio' => $row['NRCARDAPIO'],
            'cdfilial' => $row['CDFILIAL'],
            'nrorg' => $this->environment->getNrOrg(),
            'nrseqpratcar' => $row['NRSEQPRATCAR'],
        );

        $prato = $this->entityManager->getRepository('Planejamento\Entities\Compcard')->findBy($params);

        if (isset($prato[0])) {

            $this->entityManager->remove($prato[0]);
            $this->entityManager->flush();

        }
    }

    public function insertCnfcames($row, $dtrefecnfmes) {
        $row['DTREFECNFMES'] = $dtrefecnfmes;
        $row['__is_new'] = true;
        $newRequestOrigem = new DTO\Request\Row($row,"","","");
        $cnfcard = $this->crudService->getRowsToSave($newRequestOrigem, 'cnfcames');
        $this->crudService->save('cnfcames',$cnfcard); 
    }
    
    public function insertCardames($cdfilial, $cdservfili, $dtrefecnfmes) {

        $dtmescardapio = $this->dateToDateTime($dtrefecnfmes);

        $params = array(
            'cdfilial' => $cdfilial,
            'cdservfili' => $cdservfili,
            'dtmescardapio' => $dtmescardapio,
        );

        $cardames = $this->entityManager->getRepository('Planejamento\Entities\Cardames')->findBy($params);

        if (empty($cardames[0])) {

            $cardames = new \Planejamento\Entities\Cardames();
            $cardames = $this->buildCardames($cardames, $cdfilial, $cdservfili, $dtmescardapio);

            $this->entityManager->persist($cardames);
            $this->entityManager->flush();

        }

    }

    public function buildCardames($cardames, $cdfilial, $cdservfili, $dtmescardapio) {

        $cardames->setCdfilial($cdfilial);
        $cardames->setCdservfili($cdservfili);
        $cardames->setDtmescardapio($dtmescardapio);
        $cardames->setIdcardciclo('N');
        $cardames->setNrorg($this->environment->getNrOrg());
        $cardames->setDtinclusao(new \DateTime());
        $cardames->setDtultatu(new \DateTime());
        $cardames->setCdoperinclusao($this->environment->getCdOperador());

        return $cardames;
    }

    public function excluirCardadiaSemCampcard($row) {

        $messages = "";
        $params = array(
            'nrcardapio' => $row['NRCARDAPIO'],
            'cdfilial' => $row['CDFILIAL'],
            'nrorg' => $this->environment->getNrOrg(),
        );

        $pratos = $this->entityManager->getRepository('Planejamento\Entities\Compcard')->findBy($params);
        $retirada = $this->entityManager->getRepository('Geral\Entities\Retirada')->findBy($params);

        if (count($pratos) == 0 && count($retirada) == 0) {

            $this->excluirCardadia($row);
            $this->excluirCardacli($row);

        } else if (count($retirada) != 0) {

            $messages = "Dias que possuem retiradas não foram excluídos";

        }

        return $messages;

    }

    public function excluirCardadia($row) {

        $params = array(
            'nrcardapio' => $row['NRCARDAPIO'],
            'cdfilial' => $row['CDFILIAL'],
            'nrorg' => $this->environment->getNrOrg(),
        );

        $cardadia = $this->entityManager->getRepository('Planejamento\Entities\Cardadia')->findBy($params);

        if (isset($cardadia[0])) {

            $this->entityManager->remove($cardadia[0]);
            $this->entityManager->flush();

        }

    }

    public function excluirCardacli($row) {

        $params = array(
            'nrcardapio' => $row['NRCARDAPIO'],
            'cdfilial' => $row['CDFILIAL'],
            'nrorg' => $this->environment->getNrOrg(),
        );

        $cardacli = $this->entityManager->getRepository('Planejamento\Entities\Cardacli')->findBy($params);
        foreach ($cardacli as $cliente) {

          if (isset($cliente)) {

                $this->entityManager->remove($cliente);
                $this->entityManager->flush();

            }
        }

    }

    public function retornaCompcarddel($cdfilial, $cdservfili) {

        $filterCriteria = new FilterCriteria('compcarddel');
        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $filterCriteria->addCondition('CDSERVFILI', $cdservfili);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);

        return $dataSet;
    }

    public function retornaPratoExluido($cdfilial, $cdservfili, $dataCardapio, $nrseqpratcar) {

        $filterCriteria = new FilterCriteria('compcarddel');
        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $filterCriteria->addCondition('CDSERVFILI', $cdservfili);
        $filterCriteria->addCondition('DTPRATEXCL', $dataCardapio);
        $filterCriteria->addCondition('NRSEQPRATEXCL', $nrseqpratcar);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);

        return $dataSet->getRows();
    }

    public function removeCompcarddel($cdfilial, $cdservfili, $dataCardapio, $nrseqpratcar) {

        $params = array(
            'nrseqpratexcl' => $nrseqpratcar,
            'cdfilial' => $cdfilial,
            'nrorg' => $this->environment->getNrOrg(),
            'cdservfili' => $cdservfili,
            'dtpratexcl' => $this->dateToDateTime($dataCardapio),
        );

        $compcarddel = $this->entityManager->getRepository('Planejamento\Entities\Compcarddel')->findBy($params);

        if (isset($compcarddel[0])) {

            $this->entityManager->remove($compcarddel[0]);
            $this->entityManager->flush();

        }
    }

    public function insertCompcarddel($row) {

        $compcarddel = new \Planejamento\Entities\Compcarddel();
        $compcarddel = $this->buildCompcarddel($compcarddel, $row);

        $params = array(
            'nrseqpratexcl' => $compcarddel->getNrseqpratexcl(),
            'cdfilial' => $compcarddel->getCdfilial(),
            'nrorg' => $compcarddel->getNrorg(),
            'cdservfili' => $compcarddel->getCdservfili(),
            'dtpratexcl' => $compcarddel->getDtpratexcl(),
        );

        $existCompcarddel = $this->entityManager->getRepository('Planejamento\Entities\Compcarddel')->findBy($params);

        if (count($existCompcarddel) == 0) {

            $this->entityManager->persist($compcarddel);
            $this->entityManager->flush();

        }
    }

    public function buildCompcarddel($compcarddel, $row) {

        $tipoOperador = $this->retornaTipoOperador();
        $tipoOperador = $tipoOperador == 'FILIAL' ? 'F' : 'M';
        $cdprato = isset($row['CDPRATOMAT']) ? $row['CDPRATOMAT'] : $row['CDPRATO'];
        $cdfilauxprat = isset($row['CDFILAUXPRATMAT']) ? $row['CDFILAUXPRATMAT'] : $row['CDFILAUXPRAT'];

        $compcarddel->setCdfilial($row['CDFILIAL']);
        $compcarddel->setCdservfili($row['CDSERVFILI']);
        $compcarddel->setDtpratexcl($this->dateToDateTime($row['DATA']));
        $compcarddel->setNrseqpratexcl($row['NRSEQPRATCAR']);
        $compcarddel->setIdlocalexcl($tipoOperador);
        $compcarddel->setCdprato($cdprato);
        $compcarddel->setCdfilauxprat($cdfilauxprat);

        $compcarddel->setNrorg($this->environment->getNrOrg());
        $compcarddel->setDtinclusao(new \DateTime());
        $compcarddel->setDtultatu(new \DateTime());
        $compcarddel->setCdoperinclusao($this->environment->getCdOperador());
        $compcarddel->setIdativo('S');

        return $compcarddel;
    }
    
    public function validaDatasInverterPrato($row) {
        $query = "SELECT
            DTFERIFILI,
            DSFERIFILI
        FROM
            FERIFILI
        WHERE
            (CDFILIAL   = :CDFILIAL) AND
            (DTFERIFILI = :DTCARDAPIO)";
        $params = array(
            "CDFILIAL"   => $row['CDFILIAL'],
            "DTCARDAPIO" => $row['dataCardapio']
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    public function verificaCardapioDatas($row) {
        $query = "SELECT 
            NRCARDAPIO
        FROM
            CARDADIA
        WHERE
            CDFILIAL = :CDFILIAL AND
            CDSERVFILI = :CDSERVFILI AND
            DTCARDAPIO = :DTCARDAPIO";
        $paramsOrig = array(
            "CDFILIAL"   => $row['CDFILIAL'],
            "CDSERVFILI" => $row['CDSERVFILI'],
            "DTCARDAPIO" => $row['dataCardapioOrig']
        );
        $paramsDest = array(
            "CDFILIAL"   => $row['CDFILIAL'],
            "CDSERVFILI" => $row['CDSERVFILI'],
            "DTCARDAPIO" => $row['dataCardapioDest']
        );
        return [$this->entityManager->getConnection()->fetchAll($query, $paramsOrig), $this->entityManager->getConnection()->fetchAll($query, $paramsDest)];
    }
    
    public function inverterPratos($row) {
        $query = "UPDATE
            CARDADIA
        SET
            DTCARDAPIO = :DTCARDDEST
        WHERE
            CDFILIAL = :CDFILIAL AND
            CDSERVFILI = :CDSERVFILI AND
            DTCARDAPIO = :DTCARDORIG AND
            NRCARDAPIO = :NRCARDAPIO";
        $paramsOrig = array(
            "CDFILIAL"   => $row['CDFILIAL'],
            "CDSERVFILI" => $row['CDSERVFILI'],
            "DTCARDDEST" => $row['dataCardapioDest'],
            "DTCARDORIG" => $row['dataCardapioOrig'],
            "NRCARDAPIO" => $row['NRCARDORIG']
        );
        $paramsDest = array(
            "CDFILIAL"   => $row['CDFILIAL'],
            "CDSERVFILI" => $row['CDSERVFILI'],
            "DTCARDDEST" => $row['dataCardapioOrig'],
            "DTCARDORIG" => $row['dataCardapioDest'],
            "NRCARDAPIO" => $row['NRCARDDEST']
        );
        $sql = $this->entityManager->getConnection()->prepare($query);
        $sql->execute($paramsOrig);
        $sql->execute($paramsDest);
    }
    
    public function inverterPratosMantemComensais($row) {
        $query = "SELECT
                CDFILIAL,
                NRCARDAPIO,
                TO_NUMBER(NRSEQPRATCAR) NRSEQPRATCAR,
                CDFILAUXPRAT,
                CDPRATO,
                VRPECONSCARD,
                QTSOBRLIMPPR,
                CDTIPOPRAT,
                QTCOMEFETPRA,
                QTPRODUZPRAT,
                NRORDPRATCAR,
                IDPRATOESPE,
                CDFILAUXPRATMAT,
                CDPRATOMAT,
                CDOPERALTE,
                DTCOMPCARDALT
            FROM
                COMPCARD
            WHERE
                CDFILIAL   = :CDFILIAL AND
                NRCARDAPIO = :NRCARDAPIO";
        $paramsOrig = array(
            "CDFILIAL"   => $row['CDFILIAL'],
            "NRCARDAPIO" => $row['NRCARDORIG']
        );
        $paramsDest = array(
            "CDFILIAL"   => $row['CDFILIAL'],
            "NRCARDAPIO" => $row['NRCARDDEST']
        );
        $cardOrig = $this->entityManager->getConnection()->fetchAll($query, $paramsOrig);
        $cardDest = $this->entityManager->getConnection()->fetchAll($query, $paramsDest);
        $queryDelete = "DELETE FROM
                COMPCARD
            WHERE
                CDFILIAL    = :CDFILIAL AND
                NRCARDAPIO  = :NRCARDAPIO";
        
        $sql = $this->entityManager->getConnection()->prepare($queryDelete);
        $sql->execute($paramsOrig);
        $sql->execute($paramsDest);
        $queryInsert = "INSERT INTO 
            COMPCARD
                (CDFILIAL,
                NRCARDAPIO,
                NRSEQPRATCAR,
                CDFILAUXPRAT,
                CDPRATO,
                VRPECONSCARD,
                QTSOBRLIMPPR,
                CDTIPOPRAT,
                QTCOMEFETPRA,
                QTPRODUZPRAT,    
                NRORDPRATCAR,
                IDPRATOESPE,
                CDFILAUXPRATMAT,
                CDPRATOMAT,
                CDOPERALTE,
                DTCOMPCARDALT)
            VALUES
                (:P_CDFILIAL,
                :P_NRCARDAPIO,
                :P_NRSEQPRATCAR,
                :P_CDFILAUXPRAT,
                :P_CDPRATO,
                :P_VRPECONSCARD,
                :P_QTSOBRLIMPPR,
                :P_CDTIPOPRAT,
                :P_QTCOMEFETPRA,
                :P_QTPRODUZPRAT,
                :P_NRORDPRATCAR,
                :P_IDPRATOESPE,
                :P_CDFILAUXPRATMAT,
                :P_CDPRATOMAT,
                :P_CDOPERALTE,
                :P_DTCOMPCARDALT)";
        $sql = $this->entityManager->getConnection()->prepare($queryInsert);
        for($i=0; $i<sizeof($cardOrig); $i++) {
            $paramsOrig = array(
                "P_CDFILIAL"        => $cardOrig[$i]['CDFILIAL'],
                "P_NRCARDAPIO"      => $row['NRCARDDEST'],
                "P_NRSEQPRATCAR"    => $cardOrig[$i]['NRSEQPRATCAR'],
                "P_CDFILAUXPRAT"    => $cardOrig[$i]['CDFILAUXPRAT'],
                "P_CDPRATO"         => $cardOrig[$i]['CDPRATO'],
                "P_VRPECONSCARD"    => $cardOrig[$i]['VRPECONSCARD'],
                "P_QTSOBRLIMPPR"    => $cardOrig[$i]['QTSOBRLIMPPR'],
                "P_CDTIPOPRAT"      => $cardOrig[$i]['CDTIPOPRAT'],
                "P_QTCOMEFETPRA"    => $cardOrig[$i]['QTCOMEFETPRA'],
                "P_QTPRODUZPRAT"    => $cardOrig[$i]['QTPRODUZPRAT'],
                "P_NRORDPRATCAR"    => $cardOrig[$i]['NRORDPRATCAR'],
                "P_IDPRATOESPE"     => $cardOrig[$i]['IDPRATOESPE'],
                "P_CDFILAUXPRATMAT" => $cardOrig[$i]['CDFILAUXPRATMAT'],
                "P_CDPRATOMAT"      => $cardOrig[$i]['CDPRATOMAT'],
                "P_CDOPERALTE"      => $cardOrig[$i]['CDOPERALTE'],
                "P_DTCOMPCARDALT"   => $cardOrig[$i]['DTCOMPCARDALT']
            );
            $sql->execute($paramsOrig);
        }
        for($i=0; $i<sizeof($cardDest); $i++) {
            $paramsDest = array(
                "P_CDFILIAL"        => $cardDest[$i]['CDFILIAL'],
                "P_NRCARDAPIO"      => $row['NRCARDORIG'],
                "P_NRSEQPRATCAR"    => $cardDest[$i]['NRSEQPRATCAR'],
                "P_CDFILAUXPRAT"    => $cardDest[$i]['CDFILAUXPRAT'],
                "P_CDPRATO"         => $cardDest[$i]['CDPRATO'],
                "P_VRPECONSCARD"    => $cardDest[$i]['VRPECONSCARD'],
                "P_QTSOBRLIMPPR"    => $cardDest[$i]['QTSOBRLIMPPR'],
                "P_CDTIPOPRAT"      => $cardDest[$i]['CDTIPOPRAT'],
                "P_QTCOMEFETPRA"    => $cardDest[$i]['QTCOMEFETPRA'],
                "P_QTPRODUZPRAT"    => $cardDest[$i]['QTPRODUZPRAT'],
                "P_NRORDPRATCAR"    => $cardDest[$i]['NRORDPRATCAR'],
                "P_IDPRATOESPE"     => $cardDest[$i]['IDPRATOESPE'],
                "P_CDFILAUXPRATMAT" => $cardDest[$i]['CDFILAUXPRATMAT'],
                "P_CDPRATOMAT"      => $cardDest[$i]['CDPRATOMAT'],
                "P_CDOPERALTE"      => $cardDest[$i]['CDOPERALTE'],
                "P_DTCOMPCARDALT"   => $cardDest[$i]['DTCOMPCARDALT']
            );
            $sql->execute($paramsDest);
        }
    }
    
    public function getExportarCardapioParams($row) {
        $row['NRLISTA'] = "";
        $row['CDSERVFILIORIG'] = "";
        $row['CDSERVFILIDESTCONCAT'] = "";
        $query = "INSERT INTO LISTA (CDLISTA, NRLISTA)
                    SELECT LPAD(TO_CHAR(NRSEQCNFMES), 3, '0'),
                        :NRLISTA
                    FROM CNFCAMES
                    WHERE CDFILIAL = :CDFILIAL AND
                        CDSERVFILI = :CDSERVFILI AND
                        DTREFECNFMES = :DTREFECNFMES";
        foreach ($row['CDSERVFILIDEST'] as $cdlista) {
            $nrlista = $this->novoCodigo->getCodeByTableName("LISTA", $this->environment->getNrOrg());
            $row['NRLISTA'] .= $nrlista."_";
            $row['CDSERVFILIORIG'] .= $row['CDSERVFILI']."_";
            $row['CDSERVFILIDESTCONCAT'] .= $cdlista."_";
            $params = array(
                "NRLISTA"   => $nrlista,
                "CDFILIAL" => $row['CDFILIAL'],
                "CDSERVFILI" => $row['CDSERVFILI'],
                "DTREFECNFMES" => $row['DTMESCARDAPIO']
            );
            $sql = $this->entityManager->getConnection()->prepare($query);
            $sql->execute($params);
        }
        $row['MODOPROC'] = substr($this->retornaTipoOperador(), 0, 1);
        return $row;
    }
    
    public function exportarCardapio($row, $response) {
        $nrlogintcard = "";
        $connection = $this->entityManager->getConnection();
        $procedure = new StoredProcedure($connection, 'EXPORTA_CARDAPIO_CONFIG');
        $procedure->addParam(new Param('P_CDFILIORIG',  Param::PARAM_INPUT, $row['CDFILIAL'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERVORIG',  Param::PARAM_INPUT, $row['CDSERVFILIORIG'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTINIORIG',   Param::PARAM_INPUT, $row['DTINICARDAPIO'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTFINORIG',   Param::PARAM_INPUT, $row['DTFINCARDAPIO'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIDEST',  Param::PARAM_INPUT, $row['CDFILIAL'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERVDEST',  Param::PARAM_INPUT, $row['CDSERVFILIDESTCONCAT'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTINIDEST',   Param::PARAM_INPUT, $row['DTINICARDAPIO'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTFINDEST',   Param::PARAM_INPUT, $row['DTFINCARDAPIO'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CONFIGORIG',  Param::PARAM_INPUT, $row['NRLISTA'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDCOPCOMENS', Param::PARAM_INPUT, $row['COPIACOMENSAIS'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDSOBREPOR',  Param::PARAM_INPUT, $row['ACRESCSOBRE'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR',  Param::PARAM_INPUT, $row['CDOPERADOR'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_MODOPROC',    Param::PARAM_INPUT, $row['MODOPROC'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLOGINTCARD',Param::PARAM_OUTPUT, $nrlogintcard, Param::PARAM_TYPE_STR, 10));
        $procedure->addParam(new Param('P_IDGERASUBST', Param::PARAM_INPUT, null));
        $nrlogintcard = $procedure->execute();
        return $nrlogintcard;
    }
    
    public function getLogIntCard($cdfilial, $nrlogintcard) {
        $query = "SELECT
                    L.IDNUMERRO,
                    L.CDFILORIGLOG,
                    FO.NMFILIAL NMFILORIG,
                    L.CDFILDESTLOG,
                    FD.NMFILIAL NMFILDEST,
                    L.CDSERDESTLOG,
                    S.NMSERVICO,
                    L.DTCARDESTLOG,
                    L.CDPRATLOG,
                    PO.NMPRATO NMPRATORIG,
                    PD.NMPRATO NMPRATDEST
                FROM
                    SERVICO S,
                    FILIAL FO,
                    FILIAL FD,
                    PRATO PO,
                    PRATO PD,
                    LOGINTCARD L
                WHERE
                    L.NRLOGINTCARD = :NRLOGINTCARD AND
                    L.CDFILDESTLOG = :CDFILDESTLOG AND
                    L.CDPRATLOG = PO.CDPRATO (+) AND
                    L.CDFAPRAORIGL = PO.CDFILAUXPRAT (+) AND
                    L.CDPRATLOG = PD.CDPRATO (+) AND
                    L.CDFAPRADESTL = PD.CDFILAUXPRAT (+) AND
                    L.CDFILDESTLOG = FD.CDFILIAL AND
                    L.CDFILORIGLOG = FO.CDFILIAL AND
                    L.CDFILDESTLOG = S.CDFILIAL AND
                    L.CDSERDESTLOG = S.CDSERVFILI
                    ORDER BY
                    L.IDNUMERRO,
                    L.CDFILDESTLOG,
                    L.CDSERDESTLOG,
                    L.DTCARDESTLOG,
                    L.CDPRATLOG";
        $params = array(
            "CDFILDESTLOG" => $cdfilial,
            "NRLOGINTCARD" => $nrlogintcard
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    public function getServicosByFilial($cdfilial) {
        $filterCriteria = new FilterCriteria(self::SERVICO);
        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        return $dataSet->getRows();
    }
    
    public function getCardaMes($cdfilial, $cdservfili, $dtmescardapio) {
        $filterCriteria = new FilterCriteria('cardames');
        $filterCriteria->addCondition('NRORG', $this->environment->getNrOrg());
        $filterCriteria->addCondition('CDFILIAL', $cdfilial);
        $filterCriteria->addCondition('CDSERVFILI', $cdservfili);
        $filterCriteria->addCondition('DTMESCARDAPIO', $dtmescardapio);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        return $dataSet->getRows();
    }
    
    public function insertDtRefeCusto($cdfilial, $cdservfili, $dtmescardapio, $dtrefecustmes) {
        $query = "INSERT INTO CARDAMES
                    (CDFILIAL,
                    CDSERVFILI,
                    DTMESCARDAPIO,
                    DTREFECUSTMES)
                SELECT 
                    DISTINCT CDFILIAL,
                    CDSERVFILI,
                    DTMESCARD AS DTMESCARDAPIO,
                    :DTREFECUSTMES AS DTREFECUSTMES
                FROM 
                    CARDADIA CD
                WHERE CDFILIAL = :CDFILIAL AND
                    DTMESCARD = :DTMESCARDAPIO AND
                    (
                        (INSTR(:CDSERVFILI, CDSERVFILI) > 0) OR
                        (:TCDSERVFILI = 'T')
                    ) AND
                    NOT EXISTS (SELECT 
                                    CM.CDSERVFILI
                                FROM
                                    CARDAMES CM
                                WHERE CD.CDFILIAL = CM.CDFILIAL AND
                                    CD.DTMESCARD = CM.DTMESCARDAPIO AND
                                    CD.CDFILIAL = CM.CDFILIAL AND
                                    CD.CDSERVFILI = CM.CDSERVFILI)";
        $sql = $this->entityManager->getConnection()->prepare($query);
        $params = array(
            "CDFILIAL" => $cdfilial,
            "DTMESCARDAPIO" => $dtmescardapio,
            "DTREFECUSTMES" => $dtrefecustmes,
            "CDSERVFILI" => $cdservfili,
            "TCDSERVFILI" => $cdservfili
        );
        $sql->execute($params);
    }
    
    public function updateDtRefeCusto($cdfilial, $cdservfili, $dtmescardapio, $dtrefecustmes) {
        $query = "UPDATE
                    CARDAMES
                SET
                    DTREFECUSTMES = :DTREFECUSTMES
                WHERE
                    CDFILIAL = :CDFILIAL AND
                    (
                        (INSTR(:CDSERVFILI, CDSERVFILI) > 0 ) OR
                        (:TCDSERVFILI = 'T')
                    ) AND
                    DTMESCARDAPIO = :DTMESCARDAPIO";
        $sql = $this->entityManager->getConnection()->prepare($query);
        $params = array(
            "CDFILIAL" => $cdfilial,
            "DTMESCARDAPIO" => $dtmescardapio,
            "DTREFECUSTMES" => $dtrefecustmes,
            "CDSERVFILI" => $cdservfili,
            "TCDSERVFILI" => $cdservfili
        );
        $sql->execute($params);
    }
    
    public function geraHistCardaMes($row) {
    	$procedure = new StoredProcedure($this->entityManager->getConnection(), 'GERA_HISTCARDAMES');
        
        $procedure->addParam(new Param('P_CDFILIAL',    Param::PARAM_INPUT,  $row['CDFILIAL'],                    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDSERVFILI',  Param::PARAM_INPUT,  $row['CDSERVFILI'],                  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTMES',       Param::PARAM_INPUT,  $row['DTCARDAPIO'],               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDTIPOCUSTO', Param::PARAM_INPUT,  $row['CDCUSTOPLA'],                 Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DTREFECUSTO', Param::PARAM_INPUT,  $row['DTREFERENCIA'],               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRORG',       Param::PARAM_INPUT,  $this->environment->getNrOrgTrab(),  Param::PARAM_TYPE_INT));
        $procedure->addParam(new Param('P_CDOPERADOR',  Param::PARAM_INPUT,  $this->environment->getCdoperador(), Param::PARAM_TYPE_STR));
        
        $result = $procedure->execute();
        
        return $result;
    }
    
    public function existsProcedureGRAVAPRECUSTO() {
        $query = "SELECT
                    OBJECT_NAME
                FROM
                    ALL_OBJECTS
                WHERE
                    OBJECT_TYPE = 'PROCEDURE' AND
                    UPPER(OBJECT_NAME) = 'GRAVAPRECUSTO'";
        return sizeof($this->entityManager->getConnection()->fetchAll($query, []));
    }
    
    public function executeGRAVAPRECUSTO($row) {
        $connection = $this->entityManager->getConnection();
        $procedure = new StoredProcedure($connection, 'GRAVAPRECUSTO');
        $procedure->addParam(new Param('PDTINICIAL',        Param::PARAM_INPUT, $row['DTCARDAPIO'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('PDTFINAL',          Param::PARAM_INPUT, $row['DTFINAL'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('PCODTIPOCUSTO',     Param::PARAM_INPUT, '6', Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('PCODPRODINICIAL',   Param::PARAM_INPUT, '1', Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('PCODPRODFINAL',     Param::PARAM_INPUT, '3zzzzzzzzz', Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('PDTREFPESQUISA',    Param::PARAM_INPUT, $row['DTREFCUS'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('PCDFILIAL',         Param::PARAM_INPUT, $row['CDFILIAL'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('PSERVICOS',         Param::PARAM_INPUT, $row['CDSERVFILI'], Param::PARAM_TYPE_STR));
        $procedure->execute();
    }
    
    public function insertDtRefeRecCusto($cdfilial, $cdservfili, $dtmescardapio, $dtrefecustrec) {
        $query = "INSERT INTO CARDAMES
                    (CDFILIAL,
                    CDSERVFILI,
                    DTMESCARDAPIO,
                    DTREFECUSTREC)
                SELECT
                    DISTINCT CDFILIAL,
                    CDSERVFILI,
                    DTMESCARD AS DTMESCARDAPIO,
                    :DTREFECUSTREC AS DTREFECUSTREC	
                FROM
                    CARDADIA CD
                WHERE
                    CDFILIAL = :CDFILIAL AND
                    (
                        (INSTR(:CDSERVFILI, CDSERVFILI) > 0) OR
                        (:TCDSERVFILI = 'T')
                    ) AND
                    DTMESCARD = :DTMESCARDAPIO AND
                    NOT EXISTS (SELECT
                                    CM.CDSERVFILI
                                FROM
                                    CARDAMES CM
                                WHERE
                                    CD.CDFILIAL = CM.CDFILIAL AND
                                    CD.DTMESCARD = CM.DTMESCARDAPIO AND
                                    CD.CDFILIAL = CM.CDFILIAL AND
                                    CD.CDSERVFILI = CM.CDSERVFILI)";
        $sql = $this->entityManager->getConnection()->prepare($query);
        $params = array(
            "CDFILIAL" => $cdfilial,
            "DTMESCARDAPIO" => $dtmescardapio,
            "DTREFECUSTREC" => $dtrefecustrec,
            "CDSERVFILI" => $cdservfili,
            "TCDSERVFILI" => $cdservfili
        );
        $sql->execute($params);
    }
    
    public function updateDtRefeRecCusto($cdfilial, $cdservfili, $dtmescardapio, $dtrefecustrec) {
        $query = "UPDATE
                    CARDAMES
                SET
                    DTREFECUSTREC = :DTREFECUSTREC
                WHERE
                    CDFILIAL = :CDFILIAL AND
                    (
                        (INSTR(:CDSERVFILI, CDSERVFILI) > 0 ) OR
                        (:TCDSERVFILI = 'T')
                    ) AND
                    DTMESCARDAPIO = :DTMESCARDAPIO";
        $sql = $this->entityManager->getConnection()->prepare($query);
        $params = array(
            "CDFILIAL" => $cdfilial,
            "DTMESCARDAPIO" => $dtmescardapio,
            "DTREFECUSTREC" => $dtrefecustrec,
            "CDSERVFILI" => $cdservfili,
            "TCDSERVFILI" => $cdservfili
        );
        $sql->execute($params);
    }
    
    public function verificaReceitaRevisada($row) {
        $query = "SELECT
                    P.CDPRATO,
                    RC.IDRECEITA,
                    NVL(RC.DTULTATURECE, P.DTULTATUPRAT) AS DTULTATURECE
                FROM
                    (SELECT
                        CDPRATO,
                        CDFILAUXPRAT,
                        IDRECEITA,
                        MAX(DTULTATURECE) AS DTULTATURECE
                    FROM
                        (SELECT
                            CDPRATO,
                            CDFILAUXPRAT,
                            'S' IDRECEITA,
                            DTULTATURECS AS DTULTATURECE
                        FROM
                            RECESERV
                        WHERE
                            CDFILIAL =:CDFILIAL AND
                            CDSERVFILI =:CDSERVFILI AND
                            (
                                CDFILAUXPRAT =:CDFILIAL OR
                                CDFILAUXPRAT = 'P'
                            )
                        UNION ALL
                        SELECT
                            CDPRATO,
                            CDFILAUXPRAT,
                            'F' IDRECEITA,
                            DTULTATURECF AS DTULTATURECE
                        FROM
                            RECEFILI RF
                        WHERE
                            CDFILIAL =:CDFILIAL AND
                            (
                                CDFILAUXPRAT =:CDFILIAL OR
                                CDFILAUXPRAT = 'P'
                            ) AND
                            NOT EXISTS (
                                SELECT
                                    CDPRATO
                                FROM
                                    RECESERV
                                WHERE
                                    CDFILIAL =:CDFILIAL AND
                                    CDSERVFILI =:CDSERVFILI AND
                                    CDPRATO = RF.CDPRATO AND
                                    CDFILAUXPRAT = RF.CDFILAUXPRAT
                            )
                        UNION ALL
                        SELECT
                            CDPRATO,
                            CDFILAUXPRAT,
                            'P' IDRECEITA,
                            DTULTATURECP AS DTULTATURECE
                        FROM
                            RECEPADR RP
                        WHERE
                            CDFILAUXPRAT = 'P' AND
                            NOT EXISTS (
                                SELECT
                                    CDPRATO
                                FROM
                                    RECEFILI
                                WHERE
                                    CDFILIAL =:CDFILIAL AND
                                    CDFILAUXPRAT = RP.CDFILAUXPRAT AND
                                    CDPRATO = RP.CDPRATO
                            ) AND
                            NOT EXISTS (
                                SELECT
                                    CDPRATO
                                FROM
                                    RECESERV
                                WHERE
                                    CDFILIAL =:CDFILIAL AND
                                    CDSERVFILI =:CDSERVFILI AND
                                    CDFILAUXPRAT = RP.CDFILAUXPRAT AND
                                    CDPRATO = RP.CDPRATO
                            )
                    )
                    GROUP BY
                        CDPRATO,
                        CDFILAUXPRAT,
                        IDRECEITA
                ) RC,
                    (SELECT
                        CDPRATO,
                        CDFILAUXPRAT,
                        DTULTATUPRAT
                    FROM
                        PRATO
                    WHERE
                        (
                            CDFILAUXPRAT =:CDFILIAL OR
                            CDFILAUXPRAT = 'P'
                        ) AND CDNVPRATO =:CDNVPRATO
                    ) P
                WHERE
                    P.CDPRATO =:CDPRATO AND
                    P.CDFILAUXPRAT =:CDFILAUXPRAT AND
                    P.CDPRATO = RC.CDPRATO (+) AND
                    P.CDFILAUXPRAT = RC.CDFILAUXPRAT (+)";
        $params = array(
            "CDFILIAL" => $row['CDFILIAL'],
            "CDSERVFILI" => $row['CDSERVFILI'],
            "CDFILAUXPRAT" => $row['CDFILAUXPRAT'],
            "CDPRATO" => $row['CDPRATO'],
            "CDNVPRATO" => $row['CDNVPRATO']
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    public function getPercentualComens($row, $diaCardapio) {
        $query = "SELECT
                    SUM(VRPECONSCARD) AS PERCENTUAL
                FROM
                    (
                        SELECT
                            CO.CDTIPOPRAT,
                            CO.NRSEQPRATCAR,
                            CO.VRPECONSCARD
                        FROM
                            CNFCAMES CN
                            JOIN CARDADIA CA ON
                                CA.CDFILIAL = CN.CDFILIAL AND
                                CA.CDSERVFILI = CN.CDSERVFILI AND
                                SUBSTR(CA.DTCARDAPIO, 4) = SUBSTR(CN.DTREFECNFMES, 4)
                            JOIN COMPCARD CO ON
                                CA.CDFILIAL = CO.CDFILIAL AND
                                CA.NRCARDAPIO = CO.NRCARDAPIO AND
                                CN.CDTIPOPRAT = CO.CDTIPOPRAT
                        WHERE
                                CN.IDCONCNFMES = 'S' AND
                                CN.IDOPCCNFMES <> 'S' AND
                                CA.CDFILIAL =:CDFILIAL AND
                                CA.CDSERVFILI =:CDSERVFILI AND
                                CA.DTCARDAPIO =:DTCARDAPIO AND
                                NOT (
                                    CO.CDTIPOPRAT =:CDTIPOPRAT AND
                                    CO.NRSEQPRATCAR =:NRSEQPRATCAR
                                )
                            GROUP BY
                                CO.CDTIPOPRAT,
                                CO.NRSEQPRATCAR,
                                CO.VRPECONSCARD
                    )";
        $params = array(
            "CDFILIAL" => $row['CDFILIAL'],
            "CDSERVFILI" => $row['CDSERVFILI'],
            "CDTIPOPRAT" => $row['CDTIPOPRAT'],
            "NRSEQPRATCAR" => $row['NRSEQPRATCAR'],
            "DTCARDAPIO" => $diaCardapio
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    public function getPrato($row) {
        $query = "SELECT
                    CA.NRCARDAPIO,
                    CA.QTCOMESTICAR,
                    CO.CDPRATO,
                    CO.CDFILAUXPRAT,
                    CO.CDPRATOMAT,
                    CO.CDFILAUXPRATMAT,
                    CO.IDPRATOESPE
                FROM
                    COMPCARD CO
                    JOIN CARDADIA CA ON
                        CO.CDFILIAL = CA.CDFILIAL AND
                        CO.NRCARDAPIO = CA.NRCARDAPIO
                WHERE
                    CA.DTCARDAPIO = :DTCARDAPIO AND
                    CA.CDSERVFILI = :CDSERVFILI AND
                    CA.CDFILIAL = :CDFILIAL AND
                    CO.CDTIPOPRAT = :CDTIPOPRAT AND
                    CO.NRSEQPRATCAR = :NRSEQPRATCAR";
        $params = array(
            "DTCARDAPIO" => $row['DTCARDAPIO'],
            "CDSERVFILI" => $row['CDSERVFILI'],
            "CDFILIAL" => $row['CDFILIAL'],
            "CDTIPOPRAT" => $row['CDTIPOPRAT'],
            "NRSEQPRATCAR" => $row['NRSEQPRATCAR']
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    public function getCardaDia($row) {
        $query = "SELECT
                    NRCARDAPIO,
                    QTCOMESTICAR,
                    QTESTITURNO1,
                    QTESTITURNO2,
                    QTESTITURNO3,
                    QTESTITURNO4,
                    QTCLIESTICAR,
                    QTFUNESTICAR,
                    QTDOAESTICAR,
                    QTCOMEFETCAR,
                    QTCLIEFETCAR,
                    QTFUNEFETCAR,
                    QTDOAEFETCAR
                FROM
                    CARDADIA
                WHERE
                    CDFILIAL = :CDFILIAL AND
                    CDSERVFILI = :CDSERVFILI AND
                    DTCARDAPIO = :DTCARDAPIO";
        $params = array(
            "DTCARDAPIO" => $row['DTCARDAPIO'],
            "CDSERVFILI" => $row['CDSERVFILI'],
            "CDFILIAL" => $row['CDFILIAL']
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
    
    public function atualizaComensaisPrato($row) {
        $query = "UPDATE 
                    COMPCARD
                SET
                    VRPECONSCARD =:VRPECONSCARD
                WHERE
                    CDFILIAL =:CDFILIAL AND
                    NRCARDAPIO =:NRCARDAPIO AND
                    NRSEQPRATCAR =:NRSEQPRATCAR AND
                    CDTIPOPRAT =:CDTIPOPRAT";
        $params = array(
            "VRPECONSCARD" => $row['VRPECONSCARD'],
            "NRCARDAPIO" => $row['NRCARDAPIO'],
            "CDFILIAL" => $row['CDFILIAL'],
            "CDTIPOPRAT" => $row['CDTIPOPRAT'],
            "NRSEQPRATCAR" => $row['NRSEQPRATCAR']
        );
        $this->entityManager->getConnection()->prepare($query)->execute($params);
    }
    
    public function checkCustoPadrao($cdfilial, $cdservfili, $dtrefecnfmes) {
        $query = "SELECT
                    DTREFCUSPADR,
                    VRCUSTPADR
                FROM
                    CUSTPADR
                WHERE
                    CDFILIAL = :CDFILIAL AND
                    CDSERVFILI = :CDSERVFILI AND
                    DTREFCUSPADR = :DTREFERENCIA";
        $params = array(
            "CDFILIAL" => $cdfilial,
            "CDSERVFILI" => $cdservfili,
            "DTREFERENCIA" => $dtrefecnfmes
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
     public function checkCenario($cdfilial, $cdservfili, $dtrefecnfmes){
          $query = "SELECT
                    NRCENARIOCARD 
                FROM
                    CARDAMES
                WHERE
                    CDFILIAL = :CDFILIAL AND
                    CDSERVFILI = :CDSERVFILI AND
                    DTMESCARDAPIO = :DTMESCARDAPIO";
        $params = array(
            "CDFILIAL" => $cdfilial,
            "CDSERVFILI" => $cdservfili,
            "DTMESCARDAPIO" => $dtrefecnfmes
        );
        return $this->entityManager->getConnection()->fetchAll($query, $params);
    }
}