<?php
namespace Teknisa\PLA\Modules\Parametros\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\GER\Services\CrudService;
use Doctrine\DBAL\Connection;
use Exception;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Pla08800HistoricoDaUnidadeController extends \Teknisa\GER\Controllers\TeknisaCrud{
    
    protected $zhFileUploader;
    protected $fileServerURL;
    protected $dataSourceName = 'pla08800_historico_da_unidade';
    
    public function __construct( $crudService, $parameterBag, $entityManager, $zhFileUploader, $fileServerURL) {
    	$this->crudService = $crudService;
    	$this->parameterBag = $parameterBag;
        $this->entityManager = $entityManager;
        $this->zhFileUploader = $zhFileUploader;
        $this->fileServerURL = $fileServerURL;
    }

    public function getObsHistFilial (DTO\Request\Filter $request, DTO\Response $response){
        parent::find($request, $response);
        $rows = $response->getDataSets()[0]->getRows();
        $response->setDataSets(array()); // Limpa os datasets para retornar apenas o dataSet convertido com utf8
        foreach($rows as &$row){
            if($row['DSOBSHISTFILIAL'])
               $row['DSOBSHISTFILIAL'] = utf8_encode(str_replace(chr(0), "",$row['DSOBSHISTFILIAL']));
               
        }
        $response->addDataSet(new DataSource\DataSet($this->dataSourceName, $rows));
        
    }
    public function saveObserv (DTO\Request $request, DTO\Response $response){
        
        $row = $request->getRow();
        $params = array(
            "P_CDFILIAL" => $row["CDFILIAL"],
            "P_DTHISTFIL" => $row["DTHISTFIL"],
            "P_CDOPERADOR" => $row["CDOPERADOR"],
            "P_CDDESCOBSFILIAL" => $row["CDDESCOBSFILIAL"],
            "P_DSOBSHISTFILIAL" => $row["DSOBSHISTFILIAL"]
        );
        $sql = '';
        if (isset($row['__is_new']) && $row['__is_new']){
            $sql = "INSERT INTO OBSHISTFILIAL (CDFILIAL, DTHISTFIL, CDOPERADOR, CDDESCOBSFILIAL, DSOBSHISTFILIAL)  VALUES(:P_CDFILIAL, :P_DTHISTFIL, :P_CDOPERADOR, :P_CDDESCOBSFILIAL, UTL_RAW.CAST_TO_RAW(:P_DSOBSHISTFILIAL))";
        } else {
            $sql = "UPDATE OBSHISTFILIAL SET 
                        DSOBSHISTFILIAL = UTL_RAW.CAST_TO_RAW(:P_DSOBSHISTFILIAL)
                    WHERE 
                        CDFILIAL = :P_CDFILIAL AND
                        CDOPERADOR = :P_CDOPERADOR AND
                        DTHISTFIL = :P_DTHISTFIL AND
                        CDDESCOBSFILIAL = :P_CDDESCOBSFILIAL";
        }
        try {
            $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
        } catch (Exception $e){
            throw $e;
        }
    }

    //Anexo
    private function setFileFullPath($dataSet) {
		$rows = $dataSet->getRows();
		foreach ($rows as &$row) {
			if ($row["DOCUMENTOANEXO"] !== null) {
				$row["DOCUMENTOANEXOFULLPATH"] = $this->fileServerURL.$row["DOCUMENTOANEXO"];
			}
		}
		return new DataSet("pla08800_historico_da_unidade_anexo", $rows);
	}

    public function anexosFind(DTO\Request $request, DTO\Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();
        $dataSet = $this->crudService->getFilteredDataset("pla08800_historico_da_unidade_anexo", $conditions, $filterCriteria->getPage(), $filterCriteria->getPageSize());
        $newDataSet = $this->setFileFullPath($dataSet);
        $response->addDataSet($newDataSet);
    }
    
    public function uploadAnexo(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->dataSourceName = "anx_obshistfilial";
            if (isset($row["fileUpload"]) && isset($row["fileUpload"][0]) && isset($row["fileUpload"][0]["name"]) && isset($row["fileUpload"][0]["path"])) {
                $row["DSDOCUMENTOANEXO"] = $row["fileUpload"][0]["name"];
                $row["DOCUMENTOANEXO"] = $row["fileUpload"][0]["path"];
            } else {
                throw new \Exception("errorSavingFile", 1);
            }
            $dataSet = new DataSet($this->dataSourceName, array($row));
            $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
            parent::save($newRequest, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function deleteAnexo(DTO\Request $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $this->dataSourceName = "pla08800_historico_da_unidade_anexo";
            foreach ($rows as $row) {
                $this->zhFileUploader->deleteFile($row["DOCUMENTOANEXO"]);
            }
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function downloadAnexo(DTO\Request $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $file = $row["DOCUMENTOANEXOFULLPATH"];
        $name = $row["DSDOCUMENTOANEXO"];
        $nameArray = explode(".", $name);
        $type = end($nameArray);
        header('Accept-Ranges: none');
        header("Content-Type: " . $type);
        header("Content-Disposition: attachment; filename=" . $name);
        readfile($file);
        exit;
    }
    
}