<?php
namespace Teknisa\PLA\Modules\Parametros\Services;

use Teknisa\Libs\Util\XLS\FileLoader;
use Teknisa\Libs\Util\XLS\FileReader;
use Teknisa\Libs\Util\XLS\Table;
use Teknisa\Libs\Util\XLS\Exception;
use PHPExcel_Cell;
use PHPExcel_Shared_Date;
use Teknisa\Libs\Util\Environment;

use Teknisa\PLA\Modules\Prato\Services\PratoService;

class Ger08400CadastramentosAutomaticoService {
    
    protected $connection;
    protected $environment;
    
    protected $prato;
    const PRODUCT_NOT_FOUND = 'NOT_FOUND';
    
    public function __construct(Environment $environment, $connection, PratoService $prato) {
        $this->environment   = $environment;
        $this->connection = $connection;
        $this->prato = $prato;
    }
    
    public function readXls($row){
        $file = $row['FILE'];
        $columnNames = $row['CAMPOS'];
        //unset($columnNames[0]);
        $sheetsToLoad = array('Sheasdasdsadet1');
        $xlsFile = FileLoader::loadFromBase64($file['content'], $file['name']);
        if(!$xlsFile){
            throw new \Exception(self::IMPORT_LOAD_FILE_ERROR);
        }
        $worksheet = $xlsFile->getActiveSheet();
        $table = FileReader::readWorksheetTable($worksheet, $columnNames);
        
        return $table;
    }
    
    private function getCdProduto($CDARVPROD){
        $NRORG = $this->environment->getNrOrg();
        $sql = "SELECT CDPRODUTO FROM PRODUTO WHERE CDARVPROD = :CDARVPROD AND NRORG = :NRORG";
        $params = array( 
            'CDARVPROD' => $CDARVPROD,
            'NRORG'     => $NRORG
        );
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
        $produto = $stmt->fetch();
        $result = self::PRODUCT_NOT_FOUND;
        if($produto !== false){
            $result = $produto['CDPRODUTO'];
        }
        return $result; 
    }
    
    public function formatXls(Table $table, $tabela){
        $columnsReference = $table->getColumnsReference();
        $tableRows = $table->getRows();
        $columnsNumber = $table->getNumberOfColumns();
        $i = 0;
        $erros = array();
        $rowAnt = array();
        $produtos = array();
        foreach ($tableRows as $chave => &$row) {
            $chave = $chave+2;
            $row = $row['content'];
            for($column = 0; $column < $columnsNumber; $column++){
                $columnLetterIndex = PHPExcel_Cell::stringFromColumnIndex($column);
                $columnName = $columnsReference[$columnLetterIndex];
                
                try {
                    if ($tabela == 'PRATO_RECEITA_PADRAO') {
                        if ($columnName == 'CDPRODUTO') {
                            $CDARVPROD = $row[$columnName];
                            $CDPRODUTO = $produtos[$CDARVPROD] ?? $this->getCdProduto($CDARVPROD);
                            $productExists = $CDPRODUTO !== self::PRODUCT_NOT_FOUND;
                            if($productExists){
                                $produtos[$CDARVPROD] = $CDPRODUTO;
                            }
                            $row[$columnName] = $productExists ? $CDPRODUTO : '';
                            $row['CDARVPROD'] = $CDARVPROD;
                            $colunas = array(
                                'CDPRATO'   => $row['CDPRATO'],
                                "CDPRODUTOANT" => $rowAnt,
                                'CDPRODUTO' => $CDPRODUTO,
                                'SGUNIPROD' => $row['SGUNIPROD'],
                                'QTPRODUTO' => $row['QTPRODUTO']
                            );
                            
                            $valor = $this->prato->formatValue($colunas, $columnName);
                            
                            $row[$columnName] = $valor['CDPRODUTO'];
                            $row['QTPRODUTO'] = $valor['QTPRODUTO'];
                            $row['SGUNIPROD'] = $valor['SGUNIPROD'];
                            
                            if($row[$columnName] != null ){ 
                                $objectPrat = array(
                                    'CDPRATO'   => $row['CDPRATO'],
                                    'CDPRODUTO' => $row[$columnName],
                                );
                                array_push($rowAnt, $objectPrat);
                            }    
                        } else {
                            $row[$columnName] = $this->prato->formatValue($row[$columnName], $columnName);
                        }
                    } else {
                        throw new \Exception('Tabela para importação não existe.');
                    }
                } catch (\Exception $e) {
                    $lista_erros = json_decode($e->getMessage());
                    if (count($lista_erros)>1){
                        foreach ($lista_erros as $erro){
                            $erros[] = array('IDREF' => $chave, 'DESCERROR' => $erro);
                        }
                    } else array_push($erros, array('IDREF' => $chave, 'DESCERROR' => $lista_erros));
                }
                
            }
            $row["ROWNUM"] = $i;

            $i++;
        }
        return array('ROWS'=>$tableRows, 'ERROR'=>$erros);
    }
}