<?php
namespace Teknisa\PLA\Modules\Servico\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;

class Ger04100Servico extends \Teknisa\GER\Controllers\TeknisaCrud {
	
	protected $servicoService;
	protected $crudService;
	protected $dataSourceManager;
	protected $util;
	protected $dtprocessa;
	protected $dataSourceName;
	protected $dataSourceNameToSave = false;
	protected $parameterBagColumns = array();
	protected $novoCodigo;
	protected $connection;
	protected $imgPathURL;

	public function __construct($servicoService, $crudService, $dataSourceManager, $util, $novoCodigo, $connection, $imgPathURL) {
		$this->servicoService = $servicoService;
		$this->crudService = $crudService;
		$this->dataSourceManager = $dataSourceManager;
		$this->util = $util;
		$this->novoCodigo = $novoCodigo;
		$this->connection = $connection;
		$this->imgPathURL = $imgPathURL;
	}
	
	public function salvarServico(DTO\Request $request, DTO\Response $response) {
		try {
			$this->dataSourceName = 'servico';
			$this->parameterBagColumns = array();
			$this->dataSourceNameToSave = $this->crudService->getDataSourceNameToSaveByName($this->dataSourceName);
			$this->dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
			$rows = $this->crudService->getRowsToSave($request, $this->dataSourceName);
			$this->dtprocessa = $this->util->retornaDataProcessamento();
			foreach ($rows as $row) {
				$this->crudService->save($this->dataSourceName, array($row));
				if ($row['IDBLOQCONFIXPER'] == 'S') {
					$this->servicoService->deletaConsumo($row['CDFILIAL'], $row['CDSERVFILI'], $this->dtprocessa, $row['NRORG']);
				}
				if(isset($row['IDSERVTRAN']) && $row['IDSERVTRAN'] == 'N'){
					$this->servicoService->deletaEmbalagens($row);
				}
			}
		} catch (\Exception $e) {
			$response->addDataSet(new DataSet("errorInfo", array("COMPLEMENTO" => $e->getMessage())));
			$response->setError(new Error("erroSalvarDados", 500));
		}
	}

	public function retornaTiposDePrato(DTO\Request $request, DTO\Response $response) {
		$this->dataSourceName = 'ger04100_tipos_de_prato';
		$this->parameterBagColumns = array('CDFILIAL', 'CDSERVFILI', 'DTREFCUSPADR', 'NRORG');
		parent::find($request, $response);
	}

	public function retornaProximaDataCustoPadrao(DTO\Request $request, DTO\Response $response) {
		$this->dataSourceName = 'ger04100_proxima_data_custo_padrao';
		$this->parameterBagColumns = array('CDFILIAL', 'CDSERVFILI', 'NRORG');
		parent::find($request, $response);
	}

	public function removeTipoDePrato(DTO\Request $request, DTO\Response $response) {
		$this->dataSourceName = 'ger04100_tipos_de_prato';
		parent::delete($request, $response);
	}

	public function removeTodosTiposDePrato(DTO\Request $request, DTO\Response $response) {
		$row = $request->getRow();
		$this->servicoService->deletaTodosTiposDePrato($row['CDFILIAL'], $row['CDSERVFILI'], $row['DTREFCUSPADR'], $row['NRORG']);
	}

	public function removeTodosPontosEntregaCliente(DTO\Request $request, DTO\Response $response) {
		$row = $request->getRow();
		$this->servicoService->deletaTodosPontosEntregaCliente($row['CDFILIAL'], $row['CDSERVFILI'], $row['CDCLIENTE'], $row['NRORG']);
	}

	public function retornaPontosEntregaCliente(DTO\Request $request, DTO\Response $response) {
		$this->dataSourceName = 'ger04100_destfili';
		$this->parameterBagColumns = array('CDFILIAL', 'CDSERVFILI', 'NRORG', 'CDCLIENTE');
		parent::find($request, $response);
	}

	public function retornaCustoServico(DTO\Request $request, DTO\Response $response) {
		$this->dataSourceName = 'ger04100_custo_servico';
		$this->parameterBagColumns = array('CDFILIAL', 'CDSERVFILI');
		parent::find($request, $response);
	}

	public function removeProdutosFaturamento(DTO\Request $request, DTO\Response $response) {
		$this->dataSourceName = 'prodfaturam';
		$request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
		$rows = $this->crudService->getRowsToDelete($request, $this->dataSourceName);
		$dataSet = new DataSet($this->dataSourceName, (array)$rows);
		$this->crudService->delete($response, $dataSet);
	}

	public function findServico(DTO\Request\Filter $request, DTO\Response $response) {
		$filterCriteria = $request->getFilterCriteria();
		$filterCriteria->setDataSourceName("ger04100_servico");
		$filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
		$newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, array());
		$dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
		$newDataSet = $this->setImgFullPath($dataSet);
        $response->addDataSet($newDataSet);
	}

	private function setImgFullPath($dataSet) {
		$rows = $dataSet->getRows();
		foreach ($rows as &$row) {
			if ($row["FGFOTOSERVICO"] !== null) {
				$row["FGFOTOFILE"] = $row["FGFOTOSERVICO"];
				$row["FGFOTOSERVICO"] = $this->imgPathURL.$row["FGFOTOSERVICO"];
			}
		}
		return new DataSet("ger04100_servico", $rows);
	}

	public function saveImgServ(DTO\Request $request, DTO\Response $response) {
		$row = $request->getRow();
		if (isset($row["IMAGE_UPLOAD"]) && isset($row["IMAGE_UPLOAD"]["0"]) 
		&& isset($row["IMAGE_UPLOAD"]["0"]["path"]) && $row["IMAGE_UPLOAD"]["0"]["path"] !== null) {
			$row["FGFOTOSERVICO"] = $row["IMAGE_UPLOAD"]["0"]["path"];
		} else if(isset($row["FGFOTOFILE"]) && $row["FGFOTOFILE"] !== null) {
			$row["FGFOTOSERVICO"] = $row["FGFOTOFILE"];
		}
		$newDataSet = new DataSet("servico", array($row));
		$newRequest = new DTO\Request\DataSet($newDataSet, "", "", "");
		$rowsToSave = $this->crudService->getRowsToSave($newRequest, "servico");
		$this->crudService->save("servico", $rowsToSave);
	}
	
	public function exportConfigPadrao(DTO\Request $request, DTO\Response $response){
		$row = $request->getRow();
		
		try{
			
			$cdServFiliOrig = $row->get('CDSERVFILI');
			$cdFilialOrig = $row->get('CDFILIAL');
			
			$servicoOrig = $this->getServico($cdFilialOrig, $cdServFiliOrig);
			
			if(isset($row['CDFILIAL_DESTINO']) && 
					is_array($row->get('CDFILIAL_DESTINO')) && 
					sizeof($row->get('CDFILIAL_DESTINO'))>0){
				
				$cdFilialDest = $row->get('CDFILIAL_DESTINO');	
			} else {
				$cdFilialDest = 'T';
			}
			
			if(isset($row['CDSERVFILI_DESTINO']) && 
					is_array($row->get('CDSERVFILI_DESTINO')) && 
					sizeof($row->get('CDSERVFILI_DESTINO'))>0){
					
				$cdServfiliDest = $row->get('CDSERVFILI_DESTINO');	
			} else {
				$cdServfiliDest = 'T';
			}
			
			if($row->get('__INCSERVUNIDEST')=='S') $this->exportIncServUniDest($row);
			if($row->get('__COPYPARAMSFORM')=='S') $this->exportParamsForm($row, $servicoOrig);
			
			$this->servicoService->exportConfigPadrao($cdFilialOrig, $cdServFiliOrig, $cdFilialDest, $cdServfiliDest);
			
		} catch(\Exception $e){
			
			$errorMessage = $e->getMessage();
			$message = '';
			
			if (strpos($errorMessage, 'ORA-02292') !== false) {
				if (strpos($errorMessage, 'FKPLA00440CONFCARD') !== false) {
					$message = "A exportação das configurações não foi realizada. Unidade destino possui registros filhos!";
				}
			} else {
				$message = $errorMessage;
			}
			
			$response->setError(new Error($message , 500));
		}
	}
	
	private function exportParamsForm($row, $servicoOrig){
		try{
			$cdServFiliOrig = $row->get('CDSERVFILI');
			$cdFilialOrig = $row->get('CDFILIAL');
			
			
			if(isset($row['CDFILIAL_DESTINO']) && 
					is_array($row->get('CDFILIAL_DESTINO')) && 
					sizeof($row->get('CDFILIAL_DESTINO'))>0){
				
				$cdFilialDest = $row->get('CDFILIAL_DESTINO');	
			} else {
				$cdFilialDest = 'T';
			}
			
			
			if(isset($row['CDSERVFILI_DESTINO']) && 
					is_array($row->get('CDSERVFILI_DESTINO')) && 
					sizeof($row->get('CDSERVFILI_DESTINO'))>0){
				
				$cdServfiliDest = $row->get('CDSERVFILI_DESTINO');	
			} else {
				$cdServfiliDest = 'T';
			}
			
			$this->servicoService->exportParamsForm($cdServFiliOrig, $cdFilialOrig, $cdFilialDest, $cdServfiliDest, $servicoOrig);
		} catch(\Exception $e){
			throw $e;
		}
	}
	
	private function exportIncServUniDest($row){
		try{
			$cdServFiliOrig = $row->get('CDSERVFILI');
			$cdFilialOrig = $row->get('CDFILIAL');
			
			if((null !== $row->get('CDFILIAL_DESTINO')) && 
					is_array($row->get('CDFILIAL_DESTINO')) && 
					sizeof($row->get('CDFILIAL_DESTINO'))>0){
				
				$cdFilialDest = $row->get('CDFILIAL_DESTINO');
				
			} else {
				
				$cdFilialDest = 'T';
			}
			
			$this->servicoService->exportIncServUniDest($cdServFiliOrig, $cdFilialOrig, $cdFilialDest);
		} catch(\Exception $e){
			throw $e;
		}
	}
	
	private function getServico($cdFilial, $cdServfili) {
        try{
			$conditions = array(
				array(
					"columnName" => "CDFILIAL",
					"operator"   => "=",
					"value"      => $cdFilial
				),
				array(
					"columnName" => "CDSERVFILI",
					"operator"   => "=",
					"value"      => $cdServfili
				)
			);
			$this->parameterBagColumns = array("CDFILIAL", "NRORG");
			
			$filterCriteria = new FilterCriteria('ger04100_servico', $conditions);
			$filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
			
			$newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
			
			$servico = $this->dataSourceManager->findBy($newFilterCriteria)->getRows();
			return sizeof($servico)>0 ? $servico[0] : array();
        } catch(\Exception $e){
        	throw $e;
        }
	}
	
	public function autoIncludeServico(DTO\Request $request, DTO\Response $response){
		try{
			$row = $request->getRow();
			$originData = $row->get('ORIGIN_DATA');
			$targetData = $row->get('TARGET_DATA');
			
			$this->servicoService->autoIncludeServico($originData, $targetData);
        } catch(\Exception $e){
        	throw $e;
        }
	}
	
	public function getParamGeralIdutcardpadr(DTO\Request $request, DTO\Response $response){
		$Idutcardpadr = $this->servicoService->getParamGeralIdutcardpadr();
		$response->addDataSet(new DataSet('Idutcardpadr', $Idutcardpadr[0]));	
	}
	
	public function salvaEmbalagem(DTO\Request $request, DTO\Response $response){
		try{
            $this->connection->beginTransaction();

			$row = $request->getRow();
			$row['__is_new'] = true;
			if(empty($row['NRGRUPO'])) $row['NRGRUPO'] = $this->maxNrGrupo($row) + 1;	

        	foreach ($row['NMTIPOPRAT'] as $nrseqcnfcard) {
        		$row['NRCNFDISTEMBSERV'] = $this->novoCodigo->getCodeByTableName('CNFDISTEMBSERV',$row['NRORG']);
	        	$row['NRSEQCNFCARD'] = $nrseqcnfcard;
	        	$newRequest = new DTO\Request\Row($row,"","","");
	            $dataSourceName = 'cnfdistembserv';
	            $rowToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
	            $this->crudService->save($dataSourceName,$rowToSave);
        	}
           
            $this->connection->commit();
        }catch (\Exception $e) {
           $this->connection->rollBack();
           $response->setError(new Error($e->getMessage(), 500));
        }
	}
	
	public function excluiEmbalagens(DTO\Request $request, DTO\Response $response){
		try{
			$rows =  $request->getRow();
            $this->connection->beginTransaction();
			
			foreach($rows as $row){
				$data = $this->filtraPratosEmb($row);
				$this->excluiCnfDistEmbServ($data);
			}
			$this->connection->commit();
		}catch (\Exception $e) {
        	$this->connection->rollBack(); 
        	$response->setError(new Error($e->getMessage(), 500));
        } 
	}

	private function filtraPratosEmb($row){
		if(array_key_exists('NRCNFDISTEMBSERV',$row)) $nrcnfdistembserv = $row['NRCNFDISTEMBSERV'];
		else $nrcnfdistembserv = null;
		
		$params = array(
            'CDFILIAL'			=>  $row['CDFILIAL'],
        	'CDSERVFILI'		=>  $row['CDSERVFILI'],
        	'NRGRUPO'			=>  $row['NRGRUPO'],
        	'CDCNFDISTEMBSERV'  =>  $row['CDCNFDISTEMBSERV'],
        	'CDDESTFILI'		=>  $row['CDDESTFILI'],
        	'NRCNFDISTEMBSERV'  =>	$nrcnfdistembserv
        );
        $sql = "SELECT * FROM CNFDISTEMBSERV
				WHERE CDFILIAL =:CDFILIAL AND CDSERVFILI =:CDSERVFILI AND
					  ((CDDESTFILI IS NULL AND :CDDESTFILI IS NULL) OR CDDESTFILI =:CDDESTFILI) AND
    				  (:CDCNFDISTEMBSERV IS NULL OR CDCNFDISTEMBSERV =:CDCNFDISTEMBSERV) AND 
                      (:NRGRUPO IS NULL OR NRGRUPO =:NRGRUPO) AND
    				  (:NRCNFDISTEMBSERV IS NULL OR NRCNFDISTEMBSERV =:NRCNFDISTEMBSERV)";
	        
        $data = $this->connection->fetchAll($sql, $params);
        return $data;
	}
	
	private function excluiCnfDistEmbServ($data){
		$dataSourceName = "cnfdistembserv";
	    foreach ($data as $row) {
			$newRequest = new DTO\Request\Row($row,"","","");
	     	$rowToDelete = $this->crudService->getRowsToDelete($newRequest,$dataSourceName);
	     	$dataSet = new DataSet($dataSourceName, $rowToDelete);
	        $this->dataSourceManager->delete($dataSet);
		}
	}
	
	private function maxNrGrupo($row){
		$params = array(
            'CDFILIAL'	  =>  $row['CDFILIAL'],
        	'CDSERVFILI'  =>  $row['CDSERVFILI']
        );
        $sql = "SELECT MAX(NRGRUPO) AS NRGRUPO FROM CNFDISTEMBSERV WHERE CDFILIAL =:CDFILIAL AND CDSERVFILI =:CDSERVFILI";
        $data = $this->connection->fetchAll($sql, $params);
        return $data[0]['NRGRUPO'];
	}
	
	public function excluiEmbalagensConfPadrao(DTO\Request $request, DTO\Response $response){
		try{
			$rows =  $request->getRow();
            $this->connection->beginTransaction();
			
			foreach($rows as $row){
	            $query="DELETE FROM CNFDISTEMBSERV WHERE CDFILIAL =:CDFILIAL AND CDSERVFILI =:CDSERVFILI AND NRSEQCNFCARD =:NRSEQCNFCARD";
	            $this->connection->executeQuery($query,
	            array(
	            	"CDFILIAL" => $row['CDFILIAL'],
	            	"CDSERVFILI" => $row['CDSERVFILI'],
	            	"NRSEQCNFCARD" => $row['NRSEQCNFCARD']
	            	)
	            );
			}
			$this->connection->commit();
		}catch (\Exception $e) {
        	$this->connection->rollBack(); 
        	$response->setError(new Error($e->getMessage(), 500));
        } 
	}
	
	public function salvaImportExport(DTO\Request $request, DTO\Response $response){
		try{
			$row =  $request->getRow();
            $this->connection->beginTransaction();
        	$row['CDFILIAL'] = $row['CDFILIALORI'];
    		$row['CDSERVFILI'] = $row['CDSERVFILIORI'];
    		$row['CDDESTFILI'] = empty($row['CDDESTFILIORI']) ? null : $row['CDDESTFILIORI'] ;
			$row['NRGRUPO'] = null;
			$row['CDCNFDISTEMBSERV'] = null;
			
			$dataOri = $this->filtraPratosEmb($row);
			
			if(!empty($dataOri)){
				foreach ($row['CDDESTFILIDEST'] as $destino) {
					$row['CDSERVFILIDEST']  = substr($destino,0,5);
					$row['CDDESTFILIDEST']  = empty(substr($destino,5,3)) ? null : substr($destino,5,3);
					$row['CDCLIENTEDEST']	= empty(substr($destino,8,15)) ? null : substr($destino,8,15);
					$row['CDFILIAL'] = $row['CDFILIALDEST'];
		        	$row['CDSERVFILI'] = $row['CDSERVFILIDEST'];
		    		$row['CDDESTFILI'] = empty($row['CDDESTFILIDEST']) || !isset($row['CDDESTFILIDEST']) ? null : $row['CDDESTFILIDEST'] ;
		    		$row['CDCLIENTE'] = empty($row['CDCLIENTEDEST']) || !isset($row['CDCLIENTEDEST']) ? null : $row['CDCLIENTEDEST'];
					$row['NRGRUPO'] = null;
					$row['CDCNFDISTEMBSERV'] = null;
			
					$dataDest = $this->filtraPratosEmb($row);
					
					if(!empty($dataDest))$this->excluiCnfDistEmbServ($dataDest);
					$this->salvaDestino($dataOri, $row);	
					
				}
				if($row['COPIARDISTNIVSERV'] == 'S' ){
					foreach($row['NMSERVFILIDEST'] as $servico){
						$row['CDFILIAL'] = $row['CDFILIALDEST'];
		        		$row['CDSERVFILI'] = $servico;
						$row['CDDESTFILI'] = null;
		    			$row['CDCLIENTE'] = null;
		    			$dataDest = $this->filtraPratosEmb($row);
		    			if(!empty($dataDest))$this->excluiCnfDistEmbServ($dataDest);
		    			$this->salvaDestino($dataOri, $row);
					}
				}
				
			}else $response->addDataSet(new DataSet ('naoPossuiEmbalagem', array($row)));
			
			$this->connection->commit();
		}catch (\Exception $e) {
        	$this->connection->rollBack(); 
        	$response->setError(new Error($e->getMessage(), 500));
        } 
	}
	
	private function salvaDestino($dataOri, $row){
		
		$novoNrgrupo = $this->maxNrGrupo(array('CDFILIAL' => $row['CDFILIAL'], 'CDSERVFILI' => $row['CDSERVFILI'])) + 1;
		$nrgrupoOri = $dataOri[0]['NRGRUPO'];
		foreach($dataOri as $rowOri){
			if($rowOri['NRGRUPO'] != $nrgrupoOri){
				$nrgrupoOri = $rowOri['NRGRUPO'];
				$novoNrgrupo++;
			}
			
			$rowOri['__is_new'] = true;
			$rowOri['CDFILIAL'] = $row['CDFILIAL'];
			$rowOri['CDSERVFILI'] = $row['CDSERVFILI'];
			$rowOri['CDDESTFILI'] = $row['CDDESTFILI'];
			$rowOri['CDCLIENTE'] = $row['CDCLIENTE'];
			$rowOri['NRGRUPO'] = $novoNrgrupo;
			$rowOri['NRCNFDISTEMBSERV'] = $this->novoCodigo->getCodeByTableName('CNFDISTEMBSERV', $rowOri['NRORG']);
			$newRequest = new DTO\Request\Row($rowOri,"","","");
            $dataSourceName = 'cnfdistembserv';
            $rowToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
            $this->crudService->save($dataSourceName,$rowToSave);
		}
	}
	
	public function verificaCodigoExistente($request, $response){
		$row = $request->getRow();
		 $conditions = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $row['CDFILIAL']
            ),
            array(
                "columnName" => "CDSERVFILI",
                "operator" => "=",
                "value" => $row['CDSERVPADR']
            )
        );
        
        $dataset = $this->crudService->getFilteredDataSet('servico', $conditions);
        $servico = $dataset->getRows();
        
        $response->addDataSet(new DataSet ('existe', array(!empty($servico))));
	}
	
	public function deleteServFat(DTO\Request $request, DTO\Response $response){
		$dataSourceName = 'ger04100_servico_faturamento';
		$request = $this->crudService->buildDataSetIfRow($request, $dataSourceName);
		$rows = $this->crudService->getRowsToDelete($request, $dataSourceName);
		$dataSet = new DataSet($dataSourceName, (array)$rows);
		$this->crudService->delete($response, $dataSet);
	}
	
	public function retornaDtProcessaPorFilial(DTO\Request\Row $request, DTO\Response $response){
		$query = "SELECT TO_CHAR(PF.DTPROCESSA, 'DD/MM/YYYY') AS DTPROCESSA, QTDIABLOQCAR FROM PARAMFILIAL PF JOIN PARAMGERAL PG ON PG.NRORG = PF.NRORG WHERE PF.CDFILIAL = :CDFILIAL";
		$row = $request->getRow();
		$params = array(
			'CDFILIAL' => $row['CDFILIAL']
		);
		$std = $this->connection->prepare($query);
		$std->execute($params);
		$response->addDataSet(new DataSet('DATA', $std->fetch()));
	}
	
	private function getAllDietConcat($row){
		$query = "SELECT
					    LISTAGG(DIETSERV.CDDIETPAD, '_') WITHIN GROUP(
					        ORDER BY
					            DIETSERV.CDDIETPAD
					    ) AS ALL_CDDIETPAD
					FROM
					    DIETSERV,
					    DIETPADR
					WHERE
					    ( DIETSERV.CDFILIAL = :CDFILIAL )
					    AND ( DIETSERV.CDSERVFILI = :CDSERVFILI )
					    AND ( DIETSERV.CDDIETPAD = DIETPADR.CDDIETPAD )";
		$params = array(
			'CDFILIAL' => $row['CDFILIAL'],
			'CDSERVFILI' => $row['CDSERVFILI']
		);
		$std = $this->connection->prepare($query);
		$std->execute($params);
		return $std->fetch()['ALL_CDDIETPAD'];
	}
	
	public function exportaComensalCardapio(DTO\Request\Row $request, DTO\Response $response){
		$row = $request->getRow();
		if(!isset($row['CDDIETPAD']) || empty($row['CDDIETPAD'])){
			$row['LDIETA'] = $this->getAllDietConcat($row);
		} else {
			$row['LDIETA'] = implode('_', $row['CDDIETPAD']);
		}
		if($row['LDIETA']){
			$row['LDIETA'] = '_'.$row['LDIETA'].'_';
		}
		$this->servicoService->exportaComensalCardapio($row);
	}
	
	public function saveConfCardGrupoPrato($request, $response){
		$dataSourceName = 'confcardgrupoprato';

		$data = $request->getRow()['GRUPO_PRATO_EXPORT'];
		$dataBeforeChange = $request->getRow()['dataBeforeChange'];
		$changedRows = $request->getRow()['changedRows'];
		$serviceData = $request->getRow()['servData'];
		$unmarkedRows = $request->getRow()['unmarkedRows'];
		
		for ($i=0; $i < sizeof($data); $i++) {		
			$newGroupAssociation = array(
				"NRCONFCARDGRUPOPRATO" => $this->novoCodigo->getCodeByTableName('CONFCARDGRUPOPRATO', $serviceData['NRORG']),
				"CDFILIAL" =>  $serviceData['CDFILIAL'],
				"CDSERVFILI" =>  $serviceData['CDSERVFILI'],
				"NRSEQCNFCARD" =>  $serviceData['NRSEQCNFCARD'],
				"NRGRUPOPRATOSERV" =>  $data[$i],
				"NRORG" => $serviceData['NRORG'],
				"__is_new" => true
			);
			
			$newGroupAssociation = new DTO\Request\Row($newGroupAssociation,"","","");	
			$rowToSave = $this->crudService->getRowsToSave($newGroupAssociation, $dataSourceName);
			$this->crudService->save($dataSourceName,$rowToSave);
		}
		for ($i=0; $i < sizeof($unmarkedRows); $i++) {
			$newRequest = new DTO\Request\Row(array(
				"NRCONFCARDGRUPOPRATO" => $unmarkedRows[$i]
			),"","","");
			$rowToDelete = $this->crudService->getRowsToDelete($newRequest,$dataSourceName);
			$dataSet = new DataSet($dataSourceName, $rowToDelete);
			$this->dataSourceManager->delete($dataSet);
		}
	}
	
}